/*
    Filename: C:\INET\SIMTEL\DOS\simtool.cpp
    Author  : Dave Nelson
    Created : Sun 06-Dec-1998 17:24:35
    Modified: Sun 14-Feb-1999 16:05:19 - Dave Nelson

    Program to sort and select records from SIMMS.TXT simtel listings.
    Copyright (c) 1998,99 by Dave Nelson.  Permission is hereby granted
    to all and sundry to use this code in any way you see fit.

    This program was developed under MSDOS v3, using Zortech C++ v3.0r4,
    but should run on pretty much any OS with a C++ compiler.

    There are only a few things I can think of in this code that will choke
    a straight C compiler: struct datecmp has a constructor, some variable
    definitions within blocks of code, and (possibly) C++ style comments.
    I don't think there's any compiler-specific stuff in here (DOS-specific
    either, for that matter).

    The following format is expected in SIMMS.TXT:

<BLANK LINE>
Directory simtelnet/msdos/00_info/
 Filename   Type Length   Date    Description
==============================================
aareadme.txt  A    4734  970125  Information about the files in this directory
    ...
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define LINEMAX     23

struct fline {
    char fnm[14];
    char typ;
    char len[9];
    char dat[9];
    char dsc[48];
    };

void showusage(char *fnm){
    printf("\nUsage:\tSIMTOOL [fnm] [switches]"
        "\n\twhere [fnm] is an optional filename (default is %s), and"
        "\n\t[switches] are any of the following:"
        "\n\t-dOPyymmdd\tSelect records by date, where OP is any of:"
        "\n\t\tEQ\tFile date must be equal to,"
        "\n\t\tGT\t...greater than,"
        "\n\t\tLT\t...less than,"
        "\n\t\tGE\t...greater or equal,"
        "\n\t\tLE\t...less or equal,"
        "\n\t\tNE\t...or not equal to the specified date."
        "\n\t-ftext\tFind text in file description."
        "\n\t-ctext\tSame as -f, but case-sensitive."
        "\n\t-ltext\tLocation: find text in directory name."
        "\n\t-ntext\tName: find text in file name."
        "\n\t-stext\tScan: find text anywhere in output line."
        "\n\t-x\tNo pause after each screenful."
        "\n\t-p\tPure output (/dir/path/filename, nothing else)."
        "\n\t-p2\tSame as above, but include description."
        "\n\t-?\tDisplay this message.\n"
        ,fnm);
    }

char *backslash(char *path){
register char *cp = path;
    while(*cp){
        if(*cp=='/') *cp='\\';
        cp++;
        }
    return path;
    }

char * splitdate(char *d){
char buf[9]="  /  /";
    memcpy(buf,d+3,2);          // these offsets into 'd' look like one too
    memcpy(buf+3,d+5,2);        // many because the date has a leading space.
    memcpy(buf+6,d+1,2);
    memcpy(d,buf,8);
    d[8] = 0;
    return d;
    }

struct datecmp {
    int cmp;
    long v;
    datecmp *nxt;
    datecmp(int c){
        cmp = c;
        nxt = NULL;
        }
    };

int datechk(long d1, datecmp *dp){
    switch(dp->cmp){
        case 1: return d1>dp->v;
        case 2: return d1<dp->v;
        case 3: return d1>=dp->v;
        case 4: return d1<=dp->v;
        case 5: return d1==dp->v;
        case 6: return d1!=dp->v;
        }
    return 0;
    }

int pgbrk(int i, char *d){
static int cnt;
    if(i<0) cnt = 0;
    else {
        cnt += i;
        if(cnt >= LINEMAX){
            printf("\nHit  for more . . .");
            if(getch()==27) return 1;
            if(d) printf("\rDirectory:\t%s (continued)...",d);
            cnt = 0;
            }
        }
    return 0;
    }

int main(int argc, char **argv){
FILE *fp;
char buf[128], *cp, dscscan[41], fnmscan[14], dir[81];
char fnm[81] = ".\\SIMMS.TXT", dirscan[41], allscan[41];
struct fline f;
int i, csens, pause=1, purefnm;
datecmp *dp=NULL, *d2;
long size, lcnt=0L;

    csens=purefnm=0;
    pgbrk(-1, NULL);                    // initialize pgbrk
    memset(dscscan,0,sizeof(dscscan));
    memset(dirscan,0,sizeof(dirscan));
    memset(fnmscan,0,sizeof(fnmscan));
    memset(allscan,0,sizeof(allscan));
    fprintf(stderr,"\nSIMTOOL\t\tCopyright (c) 1998-1999, Dave Nelson\n");

    for(i=1; i<argc; i++){
        if(*argv[i]=='-'||*argv[i]=='/') switch(toupper(argv[i][1])){
            case 'D':
                cp = &argv[i][2];
                long wl = atol(&argv[i][4]);
                *(cp+2) = 0;
                strupr(cp);
                int wk = 0;
                if(strcmp(cp,"GT")==0) wk = 1;
                if(strcmp(cp,"LT")==0) wk = 2;
                if(strcmp(cp,"GE")==0) wk = 3;
                if(strcmp(cp,"LE")==0) wk = 4;
                if(strcmp(cp,"EQ")==0) wk = 5;
                if(strcmp(cp,"NE")==0) wk = 6;
                if(wk) {
                    if(!dp) {
                        dp = new datecmp(wk);
                        d2 = dp;
                        }
                    else {
                        d2->nxt = new datecmp(wk);
                        d2 = d2->nxt;
                        }
                    d2->v = wl;
                    }
                else {
                    fprintf(stderr,"\nInvalid comparison operator: %s",cp);
                    return 3;
                    }
                break;
            case 'C':
                csens = 1;
            case 'F':
                strncpy(dscscan,&argv[i][2],sizeof(dscscan)-1);
                if(!csens) strupr(dscscan);
                break;
            case 'L':
                strncpy(dirscan,&argv[i][2],sizeof(dirscan)-1);
                strlwr(dirscan);
                break;
            case 'S':
                strncpy(allscan,&argv[i][2],sizeof(allscan)-1);
                strlwr(allscan);
                break;
            case 'N':
                strncpy(fnmscan,&argv[i][2],sizeof(fnmscan)-1);
                strlwr(fnmscan);
                break;
            case 'X':
                pause = 0;
                break;
            case 'P':
                purefnm = (argv[i][2]=='2')? 2:1;
                break;
            case '?':
            case 'H':
                showusage(fnm);
                return 0;
            default:
                fprintf(stderr,"Unknown switch: %c", argv[i][1]);
                return 2;
            }
        else strncpy(fnm,backslash(argv[i]),sizeof(fnm)-1); // no switch: must be fnm
        }

    if( (fp=fopen(fnm,"r"))==NULL){
        fprintf(stderr,"Unable to open: %s",fnm);
        return 1;
        }

    do { fgets(buf,sizeof(buf),fp); } while(strncmp(buf,"Directory",9));
    while(!feof(fp)){
        strcpy(dir, buf+10);
        if( (cp = strrchr(dir,'\n'))!=NULL) *cp = 0;
        if(*dirscan==0||strstr(dir,dirscan)){
            if(!purefnm){
                if(pause && pgbrk(3,NULL)) return 0;
                printf("\n\nDirectory:\t%s\n",dir);
                }
            }
        fgets(buf,sizeof(buf),fp);              // Filename
        fgets(buf,sizeof(buf),fp);              // ========
        while(fgets((char *)&f,sizeof(f),fp) && !feof(fp) && *f.fnm!='\n'){
            if(*dirscan && strstr(dir,dirscan)==NULL) continue;
            if(*fnmscan && strstr(f.fnm,fnmscan)==NULL) continue;
            f.fnm[13] = f.len[8] = f.dsc[46] = 0;
            cp = strrchr(f.dsc,'\n');
            if(cp) *cp = 0;
            d2 = dp;
            while(d2){
                if(!datechk(atol(f.dat),d2)) goto bail;
                d2 = d2->nxt;
                }
            if(*dscscan){
                if(csens) {
                    if(strstr(f.dsc,dscscan)==NULL) continue;
                    }
                else {
                    strcpy(buf,f.dsc);
                    strupr(buf);
                    if(strstr(buf,dscscan)==NULL) continue;
                    }
                }
            splitdate(f.dat);
            size = atol(f.len);
            if(purefnm){
                if(purefnm==2) sprintf(buf,"\n%s%s\t%s",dir,f.fnm,f.dsc);
                else sprintf(buf,"\n%s%s",dir,f.fnm);
                }
            else sprintf(buf,"\n%s %c\t%ld\t%s %s",f.fnm,f.typ,size,f.dat,f.dsc);
            if(*allscan && (strstr(buf,allscan)==NULL)) continue;
            if(pause && pgbrk(1,purefnm? NULL:dir)) return 0;
            lcnt++;                             // okay, we have output
            fputs(buf,stdout);
bail:     ;
            }
        fgets(buf,sizeof(buf),fp);              // next directory
        }
    printf("\n%ld file(s) found.", lcnt);
    return 0;
    }
